﻿/* 
==============================================================
==============================================================

Authenticate user against Active Directory with Nagios

Authors: Alan Pipitone and Fabio Frioni 

This software is free and open source, so It has no warranty

http://www.alan-pipitone.com/

==============================================================
==============================================================
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Collections;
using System.Web.UI.WebControls;
using System.DirectoryServices;
using System.Security.Principal;
using System.IO;
using System.Diagnostics;
using System.Xml;

namespace setCookie
{
    public partial class Default : System.Web.UI.Page
    {
        //global variables
        string domain = ""; //contains the user's domain
        string nagiosGroup = ""; //contains the user's nagios group
        string ldapPath = "";
        string stringAvoidLoop = "";

        protected void Page_Load(object sender, EventArgs e)
        {
            //declare all variables
            string domainAndUserName = "";
            string userName = "";
            string print = "";
            string nagiosGroupName = "";
            bool isInLdapStrings = false;

            try
            {
                //get from webconfig nagios_url parameter
                string nagios_url = Properties.Settings.Default.nagios_url;

                //read ldap connection strings
                string[] ldapConnString = Properties.Settings.Default.ldap_strings.Split('|');

                // Obtain the authenticated user's Identity
                //WindowsPrincipal winPrincipal = (WindowsPrincipal)HttpContext.Current.User;
                domainAndUserName = HttpContext.Current.User.Identity.Name.ToString();

                /* domainAndUserName contains "domain\username" but i want only username so i have to spit
                 * the string and save only the second element (that contains the username) of the array*/
                userName = domainAndUserName.Split('\\')[1];
                domain = domainAndUserName.Split('\\')[0];


                //foreach ldap connection string
                for (int cnt = 0; cnt < ldapConnString.Length; cnt++)
                {
                    //checks if the user's domain is defined in the connection string
                    if (domain.ToLower() == ldapConnString[cnt].Remove(ldapConnString[cnt].IndexOf("=")))
                    {
                        //save the ldap connection string for the user's domain
                        ldapPath = ldapConnString[cnt].Replace(domain.ToLower() + "=", "");
                        isInLdapStrings = true;
                    }
                }

                if (isInLdapStrings == true)
                {
                    //call the method that looks for nagios group
                    searchDomainGroup(userName);
                }
                /* else the string nagiosGroup will be empty,
                 * this is because it is set only by searchDomainGroup() method */

                if (nagiosGroup != "")
                {
                    //save the group name without the nagios group prefix part
                    nagiosGroupName = nagiosGroup.Replace(Properties.Settings.Default.group_prefix, "");
                    //create a cookie that contains the nagios group (nagios will use this value as user name)
                    HttpCookie nagGroupCookie = new HttpCookie("nagios_group");
                    nagGroupCookie.Value = nagiosGroupName;
                    nagGroupCookie.Path = "/";
                    nagGroupCookie.Domain = Properties.Settings.Default.cookie_domain;
                    //myCookie.Expires = DateTime.Now.AddDays(1);
                    Response.Cookies.Add(nagGroupCookie);

                    //create a cookie that contains the AD account (nagios just print this value for your information only)
                    HttpCookie nagAccountCookie = new HttpCookie("nagios_account");
                    nagAccountCookie.Value = userName;
                    nagAccountCookie.Path = "/";
                    nagAccountCookie.Domain = Properties.Settings.Default.cookie_domain;
                    //myCookie.Expires = DateTime.Now.AddDays(1);
                    Response.Cookies.Add(nagAccountCookie);

                    //do a redirect to the nagios url that called this page. 
                    if (Request.QueryString["target"] != null)
                    {
                        /* Nagios (if you don't have the above cookies) redirects you to this web page (for the authentication) with
                         * a variable in the query string called target. target contains the url of nagios web page that
                         * redirected you to this page. For example, if you access directly to
                         * http://yourNagiosServer/nagios/cgi-bin/status.cgi?hostgroup=all&style=overview (and if you don't
                         * have the cookie) it will redirect you to:
                         * http://yourIISserver/default.aspx?target=http://yourNagiosServer/nagios/cgi-bin/status.cgi?hostgroup=all&style=overview
                         * (were default.aspx is this page). In this way this page knows where redirect you after the authentication
                         */

                        int cnt = 0;
                        string redirectUrl_string = "";

                        foreach (string s in Request.QueryString)
                        {
                            /* In the nagios url stored in "target" variable may be present some characters "&". unfortunately IIS
                                * interprets "&" as separator of other query string variables
                                * (for example,in the above url: &style=overview).
                                * If we get the value of "target" variable only, we get only
                                * http://yourNagiosServer/nagios/cgi-bin/status.cgi?hostgroup=all without &style=overview.
                                * So we have to concatenate all query string variables as follow:
                                */
                            if (cnt == 0)
                            {
                                //save in redirectUrl_string the value of query string variable "target"
                                redirectUrl_string = redirectUrl_string + Request.QueryString[s];
                            }
                            else
                            {
                                //add in the redirectUrl_string all the other query string variables (name and value) after target
                                redirectUrl_string = redirectUrl_string + "&" + s + "=" + Request.QueryString[s];
                            }
                            cnt = cnt + 1;
                        }

                        //redirect to nagios
                        Response.Redirect(redirectUrl_string,false);
                    }
                    else
                    {
                        //if query string isn't present, redirect the user to the default nagios web site (see web.config)
                        Response.Redirect(nagios_url,false);
                    }
                }
                else
                {
                    //if the current user isn't redirect by the above method, the user isn't in the nagios group
                    print = "You are not authorized";
                }

                Response.Write(print);
            }
            catch(Exception ex)
            {
                //write the error log
                writeErrorLog(ex.Message, userName, domain, ex.ToString());
                Response.Write("an error has occurred");
            }
     
            
        }


        public void searchDomainGroup(string userName)
        {
            int groupCount = 0;
            string groupName = "";

            /* if you need to specify a domain account for ldap connection use this two line:
            
               DirectoryEntry de = new DirectoryEntry(ldapPath,"domain\\username","password");
               Object obj = de.NativeObject; //this is necessary to force authentication with above credential.
            */

            //otherwise use this line:

            DirectoryEntry de = new DirectoryEntry(ldapPath);

            //construct the object DirectorySearcher
            DirectorySearcher search = new DirectorySearcher(de);

            try
            {
                //set the filter (with the current username) for the search
                search.Filter = "(cn=" + userName + ")";

                //load only the property "memberOf" because we want to see only the group of the current user
                search.PropertiesToLoad.Add("memberOf");
                
                //get the result of the search
                SearchResult result = search.FindOne();

                //counter total group of current user
                groupCount = result.Properties["memberOf"].Count;

                //loop through all group of current user
                for (int counter = 0; counter <= groupCount - 1; counter++)
                {
                    //We have to split and manipulate the string to have only the group name without the path.
                    groupName = (string)result.Properties["memberOf"][counter];
                    groupName = groupName.ToLower();
                    groupName = groupName.Split(',')[0];
                    groupName = groupName.Replace("cn=", "");

                    //avoid a "member of" loop (loop example: GROUP_A is member of GROUP_B and GROUP_B is member of GROUP_A)
                    if (stringAvoidLoop.IndexOf(groupName+";") != -1)
                    {
                        break;
                    }
                    else
                    {
                        //avoid loop
                        stringAvoidLoop = stringAvoidLoop + groupName + ";";
                    }

                    //if the current group name contains nagios group prefix we have a nagios group
                    if (groupName.IndexOf(Properties.Settings.Default.group_prefix) != -1)
                    {
                        nagiosGroup = groupName;
                        break;
                    } //if we don't have the group we have to search again
                    else if (nagiosGroup == "")
                    {
                        searchDomainGroup(groupName);
                    }
                    else //we already have nagios group, so break the cycle
                    {
                        break;
                    }

                }
            }
            catch (Exception e)
            {
                //write the error log
                writeErrorLog(e.Message, userName, domain, e.ToString());
            }
            finally
            {
                //free the memory
                search.Dispose();
                de.Dispose();
            }
            
        }

        public static void writeErrorLog(string errorMessage, string user , string domain, string stacktrace)
        {
            try
            {
                //check if log is enable
                if (Properties.Settings.Default.enable_log == "true")
                {
                    //if directory doesn't exist, create it.
                    if (!Directory.Exists(Properties.Settings.Default.log_path))
                    {
                        Directory.CreateDirectory(Properties.Settings.Default.log_path);
                    }

                    if (File.Exists(Properties.Settings.Default.log_path + "\\log.xml"))
                    {
                        //using System.Xml;
                        XmlDocument xLog = new XmlDocument();
                        //load the xml file
                        xLog.Load(Properties.Settings.Default.log_path + "\\log.xml");

                        //add EventLog node
                        XmlElement EventLog = xLog.CreateElement("EventLog");

                        //add child node
                        EventLog.AppendChild(xLog.CreateElement("Type")).InnerText = "Error";
                        EventLog.AppendChild(xLog.CreateElement("DateTime")).InnerText = DateTime.Now.ToString();
                        EventLog.AppendChild(xLog.CreateElement("User")).InnerText = user;
                        EventLog.AppendChild(xLog.CreateElement("Domain")).InnerText = domain;
                        EventLog.AppendChild(xLog.CreateElement("StackTrace")).InnerText = "\r\n      " + stacktrace.Replace("\r\n", "\r\n   ") + "\r\n    ";

                        //add above node to xml document
                        XmlNode designsets = xLog.SelectSingleNode("/logs");
                        designsets.AppendChild(EventLog);

                        //save document
                        xLog.Save(Properties.Settings.Default.log_path + "\\log.xml");

                    }
                    else
                    {
                        //using System.Xml;
                        XmlDocument xLog = new XmlDocument();

                        //set the xml encoding
                        XmlDeclaration xDecl = xLog.CreateXmlDeclaration("1.0", System.Text.Encoding.UTF8.WebName, null);
                        xLog.AppendChild(xDecl);

                        //add root node
                        XmlElement xRoot = xLog.CreateElement("", "logs", "");
                        xLog.AppendChild(xRoot);

                        //add childs node
                        XmlElement EventLog = xLog.CreateElement("EventLog");

                        EventLog.AppendChild(xLog.CreateElement("Type")).InnerText = "Error";
                        EventLog.AppendChild(xLog.CreateElement("DateTime")).InnerText = DateTime.Now.ToString();
                        EventLog.AppendChild(xLog.CreateElement("User")).InnerText = user;
                        EventLog.AppendChild(xLog.CreateElement("Domain")).InnerText = domain;
                        EventLog.AppendChild(xLog.CreateElement("StackTrace")).InnerText = "\r\n      " + stacktrace.Replace("\r\n", "\r\n   ") + "\r\n    ";

                        xRoot.AppendChild(EventLog);

                        xLog.Save(Properties.Settings.Default.log_path + "\\log.xml");
                    }

                }
            }
            catch (XmlException ex)
            {
                //if xml log isn't good, i delete it
                File.Delete(Properties.Settings.Default.log_path + "\\log.xml");
            }
            catch
            {
            }  
        }


    }
}