/*****************************************************************************
 *
 * CGIAUTH.C - Authorization utilities for Nagios CGIs
 *
 * Copyright (c) 1999-2008 Ethan Galstad (egalstad@nagios.org)
 * Last Modified:   11-30-2008
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../include/config.h"
#include "../include/common.h"
#include "../include/objects.h"

#include "../include/cgiutils.h"
#include "../include/cgiauth.h"

extern char            main_config_file[MAX_FILENAME_LENGTH];

/* patch by Fabio Frioni - 25/01/2011 */
extern char            use_ldap_filter_groupmember[MAX_FILENAME_LENGTH];
/* patch by Alan Pipitone & Fabio Frioni - 25/01/2011 */
extern char            use_cookie_filter_nagiosuser[MAX_FILENAME_LENGTH];
extern char            use_cookie_filter_domainuser[MAX_FILENAME_LENGTH];
extern char            use_cookie_domainuser[MAX_FILENAME_LENGTH];
extern char            use_cookie_authentication_site[MAX_FILENAME_LENGTH];
/* end patch Alan Pipitone & Fabio Frioni - 25/01/2011 */

extern hostgroup       *hostgroup_list;
extern servicegroup    *servicegroup_list;

extern int             use_authentication;
extern int             use_ssl_authentication;
extern int             use_ldap_authentication;
extern int             use_cookie_authentication;



/* get current authentication information */
int get_authentication_information(authdata *authinfo){
    mmapfile *thefile;
    char *input=NULL;
    char *temp_ptr=NULL;
    char *temp_act=NULL;

    /* patch by Alan Pipitone & Fabio Frioni - 25/01/2011 */
    char query_redirect[1024];

    if(authinfo==NULL)
        return ERROR;

    /* initial values... */
    authinfo->authorized_for_all_hosts=FALSE;
    authinfo->authorized_for_all_host_commands=FALSE;
    authinfo->authorized_for_all_services=FALSE;
    authinfo->authorized_for_all_service_commands=FALSE;
    authinfo->authorized_for_system_information=FALSE;
    authinfo->authorized_for_system_commands=FALSE;
    authinfo->authorized_for_configuration_information=FALSE;
    authinfo->authorized_for_read_only=FALSE;

    /* grab username from the environment... */
    if(use_ssl_authentication) {
        /* patch by Pawl Zuzelski - 7/22/08 */
        temp_ptr=getenv("SSL_CLIENT_S_DN_CN");
        }
    else if(use_cookie_authentication) {
        /* patch by Alan Pipitone & Fabio Frioni - 25/01/2011 */
           We are looking for 2 cookies:
             nagios_group       containing the "nagios contact user"
             nagios_account     containing the "domain user account"

        strcpy(use_cookie_domainuser,"");
        temp_ptr=get_cookie_groupmember(getenv("HTTP_COOKIE"),use_cookie_filter_nagiosuser);
        temp_act=get_cookie_groupmember(getenv("HTTP_COOKIE"),use_cookie_filter_domainuser);
        if(temp_ptr==NULL) {
            /* if NULL, redirect it for a new authentication */
            sprintf(query_redirect,"<SCRIPT type='text/javascript'>window.location='%s?target=http://%s%s'</SCRIPT>",use_cookie_authentication_site,getenv("HTTP_HOST"),getenv("REQUEST_URI"));
            temp_ptr = (char *) malloc(strlen(query_redirect)+1);
            strcpy(temp_ptr,query_redirect);
            }
        else{
            /* if temp_ptr not NULL, then we get cookie containing domain account too */
            sprintf(use_cookie_domainuser,"%s",temp_act);
            }
        }
    else if(use_ldap_authentication) {
        /* patch by Fabio Frioni - 25/01/2011
         * Are we using LDAP/AD Authentication method? to use this method you need to fill REMOTE_USER EnvVar with 'MemberOf' LDAP/AD groups
         * Here is a sample how to manage LDAP/AD in apache configuration
         *
         * #########################################################
         * ##  LDAP/AD - Apache Definition
         * #########################################################
         * #    AuthName "Nagios LDAP Credentials Required"
         * #    AuthType Basic
         * #    AuthzLDAPAuthoritative off
         * #    AuthBasicProvider file ldap
         * #    AuthUserFile /usr/local/nagios/etc/htpasswd.users
         * #    AuthLDAPURL "ldap://<your_ldap_ad_site/<your_distinguished_name>?<your_user>,<your_group>?sub?(objectClass=user)" NONE
         * #    AuthLDAPBindDN "<your_full_distinguished_name_user_account_if_AD_access"
         * #    AuthLDAPBindPassword "<your_password_for_AD_access>"
         * #    AuthLDAPRemoteUserIsDN off
         * #    AuthLDAPRemoteUserAttribute memberOf
         * #    require valid-user
         * #    satisfy all
         * ########################################################
         * #  End LDAP Definition
         * ########################################################
         *
         */
        temp_ptr=get_ldap_groupmember(getenv("REMOTE_USER"),use_ldap_filter_groupmember);
        }
    else{
        temp_ptr=getenv("REMOTE_USER");
        }
    if(temp_ptr==NULL){
        authinfo->username="";
        authinfo->authenticated=FALSE;
            }
    else{
        authinfo->username=(char *)malloc(strlen(temp_ptr)+1);
        if(authinfo->username==NULL)
            authinfo->username="";
        else
            strcpy(authinfo->username,temp_ptr);
        if(!strcmp(authinfo->username,""))
            authinfo->authenticated=FALSE;
        else
            authinfo->authenticated=TRUE;
    }

    /* read in authorization override vars from config file... */
    if((thefile=mmap_fopen(get_cgi_config_location()))!=NULL){

        while(1){

            /* free memory */
            free(input);

            /* read the next line */
            if((input=mmap_fgets_multiline(thefile))==NULL)
                break;

            strip(input);

            /* we don't have a username yet, so fake the authentication if we find a default username defined */
            if(!strcmp(authinfo->username,"") && strstr(input,"default_user_name=")==input){
                temp_ptr=strtok(input,"=");
                temp_ptr=strtok(NULL,",");
                authinfo->username=(char *)malloc(strlen(temp_ptr)+1);
                if(authinfo->username==NULL)
                    authinfo->username="";
                else
                    strcpy(authinfo->username,temp_ptr);
                if(!strcmp(authinfo->username,""))
                    authinfo->authenticated=FALSE;
                else
                    authinfo->authenticated=TRUE;
                    }

            else if(strstr(input,"authorized_for_all_hosts=")==input){
                temp_ptr=strtok(input,"=");
                while((temp_ptr=strtok(NULL,","))){
                    if(!strcmp(temp_ptr,authinfo->username) || !strcmp(temp_ptr,"*"))
                        authinfo->authorized_for_all_hosts=TRUE;
                        }
                    }
            else if(strstr(input,"authorized_for_all_services=")==input){
                temp_ptr=strtok(input,"=");
                while((temp_ptr=strtok(NULL,","))){
                    if(!strcmp(temp_ptr,authinfo->username) || !strcmp(temp_ptr,"*"))
                        authinfo->authorized_for_all_services=TRUE;
                        }
                    }
            else if(strstr(input,"authorized_for_system_information=")==input){
                temp_ptr=strtok(input,"=");
                while((temp_ptr=strtok(NULL,","))){
                    if(!strcmp(temp_ptr,authinfo->username) || !strcmp(temp_ptr,"*"))
                        authinfo->authorized_for_system_information=TRUE;
                        }
                    }
            else if(strstr(input,"authorized_for_configuration_information=")==input){
                temp_ptr=strtok(input,"=");
                while((temp_ptr=strtok(NULL,","))){
                    if(!strcmp(temp_ptr,authinfo->username) || !strcmp(temp_ptr,"*"))
                        authinfo->authorized_for_configuration_information=TRUE;
                        }
                    }
            else if(strstr(input,"authorized_for_all_host_commands=")==input){
                temp_ptr=strtok(input,"=");
                while((temp_ptr=strtok(NULL,","))){
                    if(!strcmp(temp_ptr,authinfo->username) || !strcmp(temp_ptr,"*"))
                        authinfo->authorized_for_all_host_commands=TRUE;
                        }
                    }
            else if(strstr(input,"authorized_for_all_service_commands=")==input){
                temp_ptr=strtok(input,"=");
                while((temp_ptr=strtok(NULL,","))){
                    if(!strcmp(temp_ptr,authinfo->username) || !strcmp(temp_ptr,"*"))
                        authinfo->authorized_for_all_service_commands=TRUE;
                        }
                    }
            else if(strstr(input,"authorized_for_system_commands=")==input){
                temp_ptr=strtok(input,"=");
                while((temp_ptr=strtok(NULL,","))){
                    if(!strcmp(temp_ptr,authinfo->username) || !strcmp(temp_ptr,"*"))
                        authinfo->authorized_for_system_commands=TRUE;
                        }
                    }
            else if(strstr(input,"authorized_for_read_only=")==input){
                                temp_ptr=strtok(input,"=");
                                while((temp_ptr=strtok(NULL,","))){
                                        if(!strcmp(temp_ptr,authinfo->username) || !strcmp(temp_ptr,"*"))
                                                authinfo->authorized_for_read_only=TRUE;
                                        }
                                }
                }

        /* free memory and close the file */
        free(input);
        mmap_fclose(thefile);
            }

    if(authinfo->authenticated==TRUE)
        return OK;
    else
        return ERROR;
        }



/* check if user is authorized to view information about a particular host */
int is_authorized_for_host(host *hst, authdata *authinfo){
    contact *temp_contact;

    if(hst==NULL)
        return FALSE;

    /* if we're not using authentication, fake it */
    if(use_authentication==FALSE)
        return TRUE;

    /* if this user has not authenticated return error */
    if(authinfo->authenticated==FALSE)
        return FALSE;

    /* if this user is authorized for all hosts, they are for this one... */
    if(is_authorized_for_all_hosts(authinfo)==TRUE)
        return TRUE;

    /* find the contact */
    temp_contact=find_contact(authinfo->username);

    /* see if this user is a contact for the host */
    if(is_contact_for_host(hst,temp_contact)==TRUE)
        return TRUE;

    /* see if this user is an escalated contact for the host */
    if(is_escalated_contact_for_host(hst,temp_contact)==TRUE)
        return TRUE;

    return FALSE;
        }


/* check if user is authorized to view information about all hosts in a particular hostgroup */
int is_authorized_for_hostgroup(hostgroup *hg, authdata *authinfo){
    hostsmember *temp_hostsmember;
    host *temp_host;

    if(hg==NULL)
        return FALSE;

    /* CHANGED in 2.0 - user must be authorized for ALL hosts in a hostgroup, not just one */
    /* see if user is authorized for all hosts in the hostgroup */
    for(temp_hostsmember=hg->members;temp_hostsmember!=NULL;temp_hostsmember=temp_hostsmember->next){
        temp_host=find_host(temp_hostsmember->host_name);
        if(is_authorized_for_host(temp_host,authinfo)==FALSE)
            return FALSE;
            }

    return TRUE;
        }



/* check if user is authorized to view information about all services in a particular servicegroup */
int is_authorized_for_servicegroup(servicegroup *sg, authdata *authinfo){
    servicesmember *temp_servicesmember;
    service *temp_service;

    if(sg==NULL)
        return FALSE;

    /* see if user is authorized for all services in the servicegroup */
    for(temp_servicesmember=sg->members;temp_servicesmember!=NULL;temp_servicesmember=temp_servicesmember->next){
        temp_service=find_service(temp_servicesmember->host_name,temp_servicesmember->service_description);
        if(is_authorized_for_service(temp_service,authinfo)==FALSE)
            return FALSE;
            }

    return TRUE;
        }

/* check if current user is restricted to read only */
int is_authorized_for_read_only(authdata *authinfo){

        /* if we're not using authentication, fake it */
        if(use_authentication==FALSE)
                return FALSE;

        /* if this user has not authenticated return error */
        if(authinfo->authenticated==FALSE)
                return FALSE;

        return authinfo->authorized_for_read_only;
        }

/* check if user is authorized to view information about a particular service */
int is_authorized_for_service(service *svc, authdata *authinfo){
    host *temp_host;
    contact *temp_contact;

    if(svc==NULL)
        return FALSE;

    /* if we're not using authentication, fake it */
    if(use_authentication==FALSE)
        return TRUE;

    /* if this user has not authenticated return error */
    if(authinfo->authenticated==FALSE)
        return FALSE;

    /* if this user is authorized for all services, they are for this one... */
    if(is_authorized_for_all_services(authinfo)==TRUE)
        return TRUE;

    /* find the host */
    temp_host=find_host(svc->host_name);
    if(temp_host==NULL)
        return FALSE;

    /* if this user is authorized for this host, they are for all services on it as well... */
    if(is_authorized_for_host(temp_host,authinfo)==TRUE)
        return TRUE;

    /* find the contact */
    temp_contact=find_contact(authinfo->username);

    /* see if this user is a contact for the service */
    if(is_contact_for_service(svc,temp_contact)==TRUE)
        return TRUE;

    /* see if this user is an escalated contact for the service */
    if(is_escalated_contact_for_service(svc,temp_contact)==TRUE)
        return TRUE;

    return FALSE;
        }


/* check if current user is authorized to view information on all hosts */
int is_authorized_for_all_hosts(authdata *authinfo){

    /* if we're not using authentication, fake it */
    if(use_authentication==FALSE)
        return TRUE;

    /* if this user has not authenticated return error */
    if(authinfo->authenticated==FALSE)
        return FALSE;

    return authinfo->authorized_for_all_hosts;
        }


/* check if current user is authorized to view information on all service */
int is_authorized_for_all_services(authdata *authinfo){

    /* if we're not using authentication, fake it */
    if(use_authentication==FALSE)
        return TRUE;

    /* if this user has not authenticated return error */
    if(authinfo->authenticated==FALSE)
        return FALSE;

    return authinfo->authorized_for_all_services;
        }


/* check if current user is authorized to view system information */
int is_authorized_for_system_information(authdata *authinfo){

    /* if we're not using authentication, fake it */
    if(use_authentication==FALSE)
        return TRUE;

    /* if this user has not authenticated return error */
    if(authinfo->authenticated==FALSE)
        return FALSE;

    return authinfo->authorized_for_system_information;
        }


/* check if current user is authorized to view configuration information */
int is_authorized_for_configuration_information(authdata *authinfo){

    /* if we're not using authentication, fake it */
    if(use_authentication==FALSE)
        return TRUE;

    /* if this user has not authenticated return error */
    if(authinfo->authenticated==FALSE)
        return FALSE;

    return authinfo->authorized_for_configuration_information;
        }


/* check if current user is authorized to issue system commands */
int is_authorized_for_system_commands(authdata *authinfo){

    /* if we're not using authentication, fake it */
    if(use_authentication==FALSE)
        return TRUE;

    /* if this user has not authenticated return error */
    if(authinfo->authenticated==FALSE)
        return FALSE;

    return authinfo->authorized_for_system_commands;
        }


/* check is the current user is authorized to issue commands relating to a particular service */
int is_authorized_for_service_commands(service *svc, authdata *authinfo){
    host *temp_host;
    contact *temp_contact;

    if(svc==NULL)
        return FALSE;

    /* if we're not using authentication, fake it */
    if(use_authentication==FALSE)
        return TRUE;

    /* if this user has not authenticated return error */
    if(authinfo->authenticated==FALSE)
        return FALSE;

    /* the user is authorized if they have rights to the service */
    if(is_authorized_for_service(svc,authinfo)==TRUE){

        /* find the host */
        temp_host=find_host(svc->host_name);
        if(temp_host==NULL)
            return FALSE;

        /* find the contact */
        temp_contact=find_contact(authinfo->username);

        /* reject if contact is not allowed to issue commands */
        if(temp_contact && temp_contact->can_submit_commands==FALSE)
            return FALSE;

        /* see if this user is a contact for the host */
        if(is_contact_for_host(temp_host,temp_contact)==TRUE)
            return TRUE;

        /* see if this user is an escalated contact for the host */
        if(is_escalated_contact_for_host(temp_host,temp_contact)==TRUE)
            return TRUE;

        /* this user is a contact for the service, so they have permission... */
        if(is_contact_for_service(svc,temp_contact)==TRUE)
            return TRUE;

        /* this user is an escalated contact for the service, so they have permission... */
        if(is_escalated_contact_for_service(svc,temp_contact)==TRUE)
            return TRUE;

        /* this user is not a contact for the host, so they must have been given explicit permissions to all service commands */
        if(authinfo->authorized_for_all_service_commands==TRUE)
            return TRUE;
            }

    return FALSE;
        }


/* check is the current user is authorized to issue commands relating to a particular host */
int is_authorized_for_host_commands(host *hst, authdata *authinfo){
    contact *temp_contact;

    if(hst==NULL)
        return FALSE;

    /* if we're not using authentication, fake it */
    if(use_authentication==FALSE)
        return TRUE;

    /* if this user has not authenticated return error */
    if(authinfo->authenticated==FALSE)
        return FALSE;

    /* the user is authorized if they have rights to the host */
    if(is_authorized_for_host(hst,authinfo)==TRUE){

        /* find the contact */
        temp_contact=find_contact(authinfo->username);

        /* reject if contact is not allowed to issue commands */
        if(temp_contact && temp_contact->can_submit_commands==FALSE)
            return FALSE;

        /* this user is a contact for the host, so they have permission... */
        if(is_contact_for_host(hst,temp_contact)==TRUE)
            return TRUE;

        /* this user is an escalated contact for the host, so they have permission... */
        if(is_escalated_contact_for_host(hst,temp_contact)==TRUE)
            return TRUE;

        /* this user is not a contact for the host, so they must have been given explicit permissions to all host commands */
        if(authinfo->authorized_for_all_host_commands==TRUE)
            return TRUE;
            }

    return FALSE;
        }


/* check if user is authorized to view information via LDAP using group member as account */
char *get_ldap_groupmember(char *distinguished_name, char *cookie_filter){
    char el_group[] = "; ";
    char dn_group[] = ",";
    char cn_group[] = "=";
    char ldap_cn[] = "CN";
    char ldap_dn[] = "DN";
    char ldap_ou[] = "OU";
    char *result = NULL;
    char *tempst = NULL;
    char *buffer = NULL;
    static char source_string[2048];

    /* if distiguished name is null, then return null */
    if(distinguished_name == NULL) return result;

    /* String isolation for non destructive strtok */
    strcpy(source_string,distinguished_name);
    
    /* check if source string is a source_string or what else */
    if( (strcasestr(source_string,ldap_cn) || strcasestr(source_string,ldap_dn) || strcasestr(source_string,ldap_ou))){
        /* we can obtain several groups  so     */
        /* tokenize the string separated by ';' */
        buffer = strtok( source_string, el_group );
        while( buffer != NULL )
        {
            /* check if cookie_filter is satisfacted */
            if( strcasestr(buffer,cookie_filter) ){
                tempst = NULL;
                tempst = (char *)malloc(strlen(buffer)+1);
                /* treat the matched group member */
                strcpy(tempst,buffer);
                buffer = NULL;
                /* now tokenize the distiguished name separated by ',' */
                buffer = strtok( tempst, dn_group );
                while( buffer != NULL )
                {
                    /* find the 'CN' token where we get the interested group name */
                    if( strcasestr(buffer,ldap_cn) ) {
                        tempst = NULL;
                        tempst = (char *)malloc(strlen(buffer)+1);
                        strcpy(tempst,buffer);
                        buffer = NULL;
                        /* read the first unwanted token 'CN' */
                        buffer = strtok( tempst, cn_group );
                        /* set to result the wanted token (group member) */
                        result = strtok( NULL, cn_group );
                        /* buffer to NULL, to get out of all nested 'while'  */
                        buffer = NULL;
                    } else {
                        /* otherwise analyze the next token ',' until NULL is reached */
                        buffer = strtok( NULL, dn_group );
                    }
                }
            } else {
                /* otherwise analyze the next token ';' until NULL is reached */
                buffer = strtok( NULL, el_group );
            }
        }
    } else {
        /* it seems not a distiguished name, it could be the result */
        result = NULL;
        if( *cookie_filter != '\0' ){
            result = (char *)malloc(strlen(source_string)+1);
            strcpy(result,source_string);

        }
    }
    return result;
}    


/* check if user is authorized to view information via HTTP_COOKIE using group member as account */
char *get_cookie_groupmember(char *coockies_list, char *cookie_filter){
    char el_group[] = "; ";
    char cn_group[] = "=";
    char *result = NULL;
    char *tempst = NULL;
    char *buffer = NULL;
    static char source_string[2048];

    /* No HTTP_COOKIE, then require again authentication */
    if(coockies_list == NULL) return result;

    /* String isolation for non destructive strtok */
    strcpy(source_string,coockies_list);
    
    /* check if source string is a cookies_list or what else */
    if( strcasestr(source_string,cookie_filter) ){
        /* we can obtain several groups  so     */
        /* tokenize the string separated by ';' */
        buffer = (char *) malloc(strlen(source_string)+1);
        buffer = strtok( source_string, el_group );
        while( buffer != NULL )
        {
            /* check if cookie_filter is satisfacted */
            if( strcasestr(buffer,cookie_filter) ){
                tempst = NULL;
                tempst = (char *)malloc(strlen(buffer)+1);
                /* treat the matched group member */
                strcpy(tempst,buffer);
                buffer = NULL;
                /* read the first unwanted token 'CN' */
                buffer = strtok( tempst, cn_group );
                /* set to result the wanted token (group member) */
                result = strtok( NULL, cn_group );
                /* buffer to NULL, to get out of all nested 'while'  */
                buffer = NULL;
            } else {
                /* otherwise analyze the next token ';' until NULL is reached */
                buffer = strtok( NULL, el_group );
            }
        }
     }
     return result;
}
