<?php
/*****************************************************************************
 *
 * CoreModLogonCookie.php - Based on modules created by Lars Michelsen <lars@vertical-visions.de>
 *
 * Copyright (c) 2004-2010 NagVis Project (Contact: info@nagvis.org)
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/
 
/**
 * Based on the work of Lars Michelsen <lars@vertical-visions.de>
 * Authors: Fabio Frioni <fabio.frioni@gmail.com>
 *
 * This module is intended to be used with COOKIE authentication method
 * by reading the HTTP_COOKIE environment variable.
 * To activate this module you must edit the "nagvis.ini.php" config file
 * and put the following definitions:
 *
 *      logonmodule="LogonCookie"
 *      logonenvvar="nagios_group"
 *
 * where logonmodule is this module and logonenvvar is the name of cookie
 * which contains the user name
 *
 * HTTP_COOKIE Header variable containing "nagios_group=user_name" cookie
 *
 */
class CoreModLogonCookie extends CoreModule {
	protected $CORE;
	private   $bVerbose;

	public function __construct($CORE) {
		$this->sName = 'LogonCookie';
		$this->CORE = $CORE;
		
		$this->aActions = Array('view' => 0);
		$this->bVerbose = true;
	}

	public function beQuiet() {
		// Sometimes less is more...
		$this->bVerbose = false;
	}
	
	public function handleAction() {
		$sReturn = '';
		
		if($this->offersAction($this->sAction)) {
			switch($this->sAction) {
				case 'view':
					// Check if user is already authenticated
					if(!isset($this->AUTHENTICATION) || !$this->AUTHENTICATION->isAuthenticated()) {
						$logonev = $this->CORE->getMainCfg()->getValue('global', 'logonenvvar');
						if(!isset($_SERVER['HTTP_COOKIE']) || preg_match('/'.$logonev.'/i',$_SERVER['HTTP_COOKIE'])==0){
                            // NOTE!!!! this part is EMBEDDED, so you MUST change/manage by yourself
                            // for compatibility we decided to leave this setting as is
                            //
                            // Explanation: this is a redirection for your favorite authentication website
                            // which will provide the right cookie for the user browser.
                            //
                            // For instance, we are appending the nagvis requested URL and bringing it
                            // to an IIS website for SSO. After the user is authenticated, the
                            // authentication website will redirect again to previous URL defined in 'target' parameter
							$this->redirecturl('http://<your_server_for_authentication>?target='. $this->curPageURL(),2);
							exit;
						}

						
						// Get environment variable to use
						$sEnvVar = $_SERVER['HTTP_COOKIE'];
						
						// Check cookie contains the user
						$sUser = $this->get_cookie_groupmember($sEnvVar, $this->CORE->getMainCfg()->getValue('global', 'logonenvvar'));
						if($sUser !== '') {
							// $sUser = $_SERVER[$sEnvVar];
							
							// Get the authentication instance from the core
							$this->AUTHENTICATION = $this->CORE->getAuthentication();
							
							// Check if the user exists
							if(!$this->AUTHENTICATION->checkUserExists($sUser)) {
								$bCreateUser = $this->CORE->getMainCfg()->getValue('global', 'logonenvcreateuser');
								if(settype($bCreateUser, 'boolean')) {
									// Create user when not existing yet
									// Important to add a random password here. When someone
									// changes the logon mechanism to e.g. LogonDialog it
									// would be possible to logon with a hardcoded password
									$this->AUTHENTICATION->createUser($sUser, (time() * rand(1, 10)));
									
									$role = $this->CORE->getMainCfg()->getValue('global', 'logonenvcreaterole');
									if($role !== '') {
										$AUTHORISATION = new CoreAuthorisationHandler($this->CORE, $this->AUTHENTICATION, $this->CORE->getMainCfg()->getValue('global', 'authorisationmodule'));
										$AUTHORISATION->parsePermissions();
										$AUTHORISATION->updateUserRoles($AUTHORISATION->getUserId($sUser), Array($AUTHORISATION->getRoleId($role)));
									}
								} else {
									if($this->bVerbose) {
										new GlobalMessage('ERROR', $this->CORE->getLang()->getText('Unable to authenticate user. User does not exist.'));
									}
									return false;
								}
							}
							
							// Authenticate the user without providing logon information
							
							// Reset the authentication check. Without this the cached result
							// would prevent the authentication check with the given credentials
							$this->AUTHENTICATION->resetAuthCheck();
							
							// Set credentials to authenticate
							// Use dummy password - with empty or unset password the session
							// auth module would not authenticate the user
							$this->AUTHENTICATION->passCredentials(Array('user' => $sUser, 'password' => '.'));
							
							// Try to authenticate the user
							if($this->AUTHENTICATION->isAuthenticated(AUTH_TRUST_USERNAME)) {
								// Redirect without message to the user
								header('Location:'.CoreRequestHandler::getRequestUri($this->CORE->getMainCfg()->getValue('paths', 'htmlbase')));
							} else {
								// Invalid credentials
								// FIXME: Count tries and maybe block somehow
								if($this->bVerbose) {
									new GlobalMessage('ERROR', $this->CORE->getLang()->getText('You entered invalid credentials.'), null, $this->CORE->getLang()->getText('Authentication failed'), 10, CoreRequestHandler::getReferer(''));
								}
								
								return false;
							}
						} else {
							if($this->bVerbose) {
								new GlobalMessage('ERROR', $this->CORE->getLang()->getText('Unable to authenticate user. The environment variable [VAR] is not set or empty.', Array('VAR' => htmlentities($sUser))));
							}

							return false;
						}
					} else {
						// When the user is already authenticated redirect to start page (overview)
						header('Location:'.CoreRequestHandler::getRequestUri($this->CORE->getMainCfg()->getValue('paths', 'htmlbase')));
					}
				break;
			}
		}
		
		return $sReturn;
	}

	public function get_cookie_groupmember($http_cookie, $contact_filter) {
		$sReturn = $this->CORE->getMainCfg()->getValue('global', 'logonenvcreaterole');
		$thisMatch = "/" . $contact_filter . "/i";
		$foundCookie = "";

		if (preg_match($thisMatch, $http_cookie)) {
			$aCookies = explode(";",$http_cookie);
			foreach($aCookies as $thisCookie) {
				if (preg_match($thisMatch, $thisCookie)) {
					$foundCookie = explode("=",$thisCookie);
					$sReturn = $foundCookie[1];
				}
			}
		}
		return $sReturn;
	}
	
        public function redirecturl($url, $seconds = FALSE) {
                if (!headers_sent() && $seconds == FALSE) {
                        header("Location: " . $url);
                } else {
                        if ($seconds == FALSE) {
                                $seconds = "0";
                        }
                        echo '<meta http-equiv="refresh" content="'.$seconds.';url='.$url.'">';
                }
                exit;
        }

        public function curPageURL() {
                $pageURL = 'http';
                if ($_SERVER["HTTPS"] == "on") {$pageURL .= "s";}
                $pageURL .= "://";
                if ($_SERVER["SERVER_PORT"] != "80") {
                        $pageURL .= $_SERVER["SERVER_NAME"].":".$_SERVER["SERVER_PORT"].$_SERVER["REQUEST_URI"];
                } else {
                        $pageURL .= $_SERVER["SERVER_NAME"].$_SERVER["REQUEST_URI"];
                }
                return $pageURL;
        }
}

?>
